#   Copyright (c) 2020 Boulder Real Time Technologies, Inc.
#
#   This software module is wholly owned by Boulder Real Time
#   Technologies, Inc. Any use of this software module without
#   express written permission from Boulder Real Time Technologies,
#   Inc. is prohibited.
#
#   Author: Ambli LLC (ambli.ll.company@gmail.com)

import connexion
import logging
import six
import os
from dateutil import parser as dt_parser
from datetime import datetime, timezone
import tempfile
from http import HTTPStatus
import sys
from flask import make_response

sys.path.append(os.environ['ANTELOPE'] + "/data/python")

from datetime import datetime
from antelope import stationxml

from webservice_fdsn.app.core import constants
from webservice_fdsn.app.utils import pf_helper
from webservice_fdsn.app.utils import utilities
from webservice_fdsn.app.utils import exceptions as apiutil_exceptions
from webservice_fdsn.app.core.station_helpers import db2stationxml_postprocess_raw_xml as xml_postprocess

log = logging.getLogger(__name__)


def get_station(starttime, network, station, channel, location,
                minlatitude, maxlatitude, minlongitude, maxlongitude,
                level=None, endtime=None, nodata=None):  # noqa: E501
    """Controller handling all station API queries

    :param starttime: Limit to metadata epochs starting on or after the specified start time (UTC)
    :type starttime: str
    :param network: Select one or more network codes. Can be SEED network codes or data center defined
                    codes. Multiple codes are comma-separated.
    :type network: str
    :param station: Select one or more SEED station codes. Multiple codes are comma-separated.
    :type station: str
    :param channel: Select one or more SEED channel codes. Multiple codes are comma-separated.
    :type channel: str
    :param location: Select one or more SEED location identifiers. Multiple identifiers are comma- separated.
                     As a special case -- (two dashes) will be translated to a string of two
                     space characters to match blank location IDs.
    :type location: str
    :param minlatitude: Limit to stations with a latitude larger than or equal to the specified minimum.
    :type minlatitude: float
    :param maxlatitude: Limit to stations with a latitude smaller than or equal to the specified maximum.
    :type maxlatitude: float
    :param minlongitude: Limit to stations with a longitude larger than or equal to the specified minimum.
    :type minlongitude: float
    :param maxlongitude: Limit to stations with a longitude smaller than or equal to the specified maximum.
    :type maxlongitude: float
    :param level: Specify the level of detail for the results.
    :type level: str
    :param endtime: Limit to metadata epochs ending on or before the specified end time (UTC)
    :type endtime: str
    :param nodata: Select status code for no data, either &#x27;204&#x27; (default) or &#x27;404&#x27;.
    :type nodata: str

    :rtype: None
    """
    log.info(f'Querying station data using params: starttime: {starttime}, network: {network}, station: {station}, '
             f'channel: {channel}, location: {location}, minlatitude: {minlatitude}, maxlatitude: {maxlatitude}, '
             f'minlongitude: {minlongitude}, maxlongitude: {maxlongitude}, level: {level}, endtime: {endtime}')
    starttime = dt_parser.parse(starttime)
    starttime = starttime.replace(tzinfo=None)
    if endtime:
        endtime = dt_parser.parse(endtime)
        endtime = endtime.replace(tzinfo=None)

    if level == constants.channel_constants.NETWORK:
        level = stationxml.NETWORK_LEVEL
    elif level == constants.channel_constants.STATION:
        level = stationxml.STATION_LEVEL
    elif level == constants.channel_constants.CHANNEL:
        level = stationxml.CHANNEL_LEVEL
    # If LOCATION or no value specified, default to RESPONSE_LEVEL
    else:
        level = stationxml.RESPONSE_LEVEL

    # Get scratch dir and dump intermediate xml there
    scratch_dir_path = pf_helper.get_project_scratch_dir()

    # Get database file/s on disk
    try:
        db_paths = utilities.get_db_files(constants.service_interfaces_constants.STATION,
                                          scratch_dir_path, starttime, endtime)
    except (apiutil_exceptions.RequiredPfOptionMissing,
            apiutil_exceptions.InvalidPfValue) as ex:
        log.error(f"Service configuration error: {ex}")
        resp = make_response(f'{ex}', HTTPStatus.INTERNAL_SERVER_ERROR)
        return resp

    if not db_paths:
        resp = make_response("", nodata)

    log.debug(f'Searching station data in {len(db_paths)} files')

    # TODO: Alaska is primary beta tester for dbcluster support, and they have station as a single file.
    # Proceed with this assumption for now to deliver next patch, but continue working towards properly
    # supporting extraction of data from multiple station dbs
    db_path = db_paths[0]
    try:
        data = stationxml.db2stationxml(db_path,
                                        pf="db2stationxml",
                                        net_regexp=network,
                                        sta_regexp=station,
                                        loc_regexp=location,
                                        cha_regexp=channel,
                                        detail=level)
    # TODO: Handle utility specific exceptions
    except Exception as ex:
        return make_response(f'{ex}', HTTPStatus.INTERNAL_SERVER_ERROR)

    if not data:
        resp = make_response("", nodata)
        return resp

    # StationXML data available. Now apply other required arg geographic and time window constraints
    with tempfile.NamedTemporaryFile(dir=scratch_dir_path, delete=True,
                                     prefix='source_', suffix='.xml') as source_xml:
        source_xml.write(data.encode('utf8'))
        source_xml.flush()
        with tempfile.NamedTemporaryFile(dir=scratch_dir_path, delete=True,
                                         prefix='dest_', suffix='.xml') as dest_xml:
            data_found = xml_postprocess.db2stationxml_filter(source_xml.name, dest_xml.name,
                                                              float(minlatitude), float(maxlatitude),
                                                              float(minlongitude), float(maxlongitude),
                                                              starttime, endtime)
            t2 = datetime.now()
            dest_xml.flush()
            if data_found:
                resp = make_response(dest_xml.read(), HTTPStatus.OK)
                return resp
            else:
                resp = make_response("", nodata)
                return resp


def get_station_version():  # noqa: E501
    """get_station_version

     # noqa: E501


    :rtype: None
    """
    return '1.2.0'


def get_station_wadl():  # noqa: E501
    """get_station_wadl

     # noqa: E501


    :rtype: None
    """
    wadl_path = os.path.join(os.path.dirname(__file__), os.path.pardir, "wadl", "station.wadl")
    with open(wadl_path, "r") as f:
        resp = make_response("".join(f.readlines()), HTTPStatus.OK)
        return resp
