#!/opt/antelope/python3.6.5/bin/python3

import os
import sys

import signal

signal.signal(signal.SIGINT, signal.SIG_DFL)

sys.path.append(os.environ['ANTELOPE'] + "/data/python")

sys.path.append(os.environ['ANTELOPE'] + "/contrib/data/python")

from webservice_fdsn.app.core.event_exporter.export_events_helpers import exceptions

#!/usr/bin/env python
'''
QuakeML export infrastructure for Antelope
------------------------------------------
This code exports events by evid from an Antelope Datascope database
into QuakeML format. For this we *most* have an event table present.

Currently only the export of a single event at a time is supported by this
interface, but export_events module does support exporting of multiple events.

The XSD files describing the schema are included in the distro
and referenced in the parameter file. There are validators that
you can use to verify your exports.

The code will send all output to STDOUT or to a file if you are
using the -o FILENAME flag at runtime. Run with an EVID to export
that single event. Need to develop a method to convert all events
in a database.

Contributors:
Mark Williams, markwilliams@seismo.unr.edu
    https://github.com/NVSeismoLab/qmlutil
Juan Reyes, reyes@ucsd.edu
Nick Ackerley, nicholas.ackerley@canada.ca

XML parser:
    XMLTODICT.PY = Parse the given XML input and convert it into a dictionary.
    #Copyright (C) 2012 Martin Blech and individual contributors.
'''
# pylint: disable=logging-not-lazy

import os
import sys
import argparse
import logging
from logging.config import dictConfig
from pkg_resources import get_distribution

try:
    from antelope import stock
except ImportError as ex:
    sys.exit("Import Error: [%s] Do you have ANTELOPE installed correctly?" %
             ex)

from webservice_fdsn.app.utils import utilities
try:
    from webservice_fdsn.app.core.event_exporter.export_events_helpers.functions import open_verify_pf, safe_pf_get
    from webservice_fdsn.app.core.event_exporter.export_events_helpers.db_reader import DatabaseReader
    from webservice_fdsn.app.core.event_exporter.export_events_helpers.css2qml import Css2Qml
    from webservice_fdsn.app.core.event_exporter.export_events_helpers.xmltodict import xmlencode
except ImportError as ex:
    sys.exit("[%s] Error loading  qml functions." % ex)


try:
    from lxml import etree
    VALIDATION_POSSIBLE = True
except ImportError:
    VALIDATION_POSSIBLE = False

logger = logging.getLogger(__name__)

_DEFAULT_SCHEMA_PATH = os.path.join(
    os.environ['ANTELOPE'], 'contrib', 'data', 'quakeml')
_DEFAULT_SCHEMA = 'QuakeML-1.2.rng'


def _argparser():
    """
    Process command-line arguments.
    """
    class MyFormatter(argparse.ArgumentDefaultsHelpFormatter,
                      argparse.RawDescriptionHelpFormatter):
        """
        Preserve linefeeds in docstring and include default values in help.
        """
        pass

    class MyParser(argparse.ArgumentParser):
        """
        Trigger printing of help on any argument parsing error.
        """
        def __init__(self, *args, **kwargs):
            super(MyParser, self).__init__(args, kwargs)

        def error(self, message):
            sys.stderr.write('error: %s\n' % message)
            self.print_help()
            sys.exit(2)

    # pylint: disable=no-member
    version = get_distribution('export_events').version
    parser = MyParser(description=__doc__,
                      formatter_class=MyFormatter,
                      version='export_events %s' % version)
    parser.add_argument('database',
                        help='path to Antelope database or descriptor')
    parser.add_argument('evid', type=int,
                        help='Antelope event identifier')
    parser.add_argument('-o', '--output_file', default=None,
                        help='save output to file')
    parser.add_argument('-s', '--schema',
                        default=os.path.join(_DEFAULT_SCHEMA_PATH,
                                             _DEFAULT_SCHEMA),
                        help='XML Schema Definition to implement')
    parser.add_argument('-p', '--pf', default='event2qmlhack.pf',
                        help='Antelope-style parameter file')
    parser.add_argument('-l', '--log', default='WARNING',
                        help='Console logging level: DEBUG, INFO, WARNING')
    return parser


def build_dbreader_options(pf_file, override_options=None):
    """
    Build options to pass to the db reader based on the passed in pf file
    :param pf_file: path to parameter file with default values
    :param override_options: user defined options which will override defaults ones from parameter file
    :rtype dict
    :return: dictionary of db open options
    """
    pf_file = stock.pffiles(pf_file)[-1]
    logger.debug("Parameter file to use [%s]" % pf_file)
    pf_object = open_verify_pf(pf_file, 1472083200)

    dbreader_options = dict()
    dbreader_options['magnitude_type_subset'] = safe_pf_get(pf_object,
                                                            'magnitude_type_subset', ['.*'])
    for x in ['mt_auth_select', 'mt_auth_reject',
              'event_auth_select', 'event_auth_reject',
              'netmag_auth_select', 'netmag_auth_reject',
              'fplane_auth_select', 'fplane_auth_reject',
              'origin_auth_select', 'origin_auth_reject',
              'arrival_auth_select', 'arrival_auth_reject',
              'detection_state_select', 'detection_state_reject']:
        if override_options and x in override_options:
            logger.debug("User requested to override \"{}\" option".format(x))
            dbreader_options[x] = override_options[x]
        else:
            dbreader_options[x] = filter(None, safe_pf_get(pf_object, x, []))
    return dbreader_options


def build_csstoqml_options(pf_file, override_options=None):
    """
    Build the csstoqml options using the passed in pf_file
    :param pf_file: path to parameter file with default values
    :param override_options: user defined options which will override defaults ones from parameter file
    :rtype dict
    :return: dictionary of options to run csstoqml with
    """
    pf_file = stock.pffiles(pf_file)[-1]
    logger.debug("Parameter file to use [%s]" % pf_file)
    pf_object = open_verify_pf(pf_file, 1472083200)

    csstoqml_options = dict()
    csstoqml_options['reviewed_flags'] = safe_pf_get(pf_object, 'reviewed_flags', ['r*', 'y*'])
    csstoqml_options['automatic_authors'] = safe_pf_get(pf_object, 'reviewed_flags',
                                                        ['oa*', 'orbassoc'])
    csstoqml_options['etype_type_map'] = safe_pf_get(pf_object, 'etype_type_map', {})
    csstoqml_options['etype_certainty_map'] = safe_pf_get(pf_object, 'etype_certainty_map', {})
    csstoqml_options['uri_prefix'] = safe_pf_get(pf_object, 'uri_prefix', 'quakeml')
    csstoqml_options['agency_uri'] = safe_pf_get(pf_object, 'agency_uri', 'local')
    csstoqml_options['agency_id'] = safe_pf_get(pf_object, 'agency_id', 'xx')
    csstoqml_options['default_network'] = safe_pf_get(pf_object, 'default_network', 'XX')
    csstoqml_options['catalog_author'] = safe_pf_get(pf_object, 'catalog_author',
                                                     'antelope.event2qml')
    csstoqml_options['qml_ns'] = safe_pf_get(pf_object, 'Q_NAMESPACE',
                                             'http://quakeml.org/xmlns/quakeml/1.2')
    csstoqml_options['anss_catalog_ns'] = safe_pf_get(pf_object, 'CATALOG_NAMESPACE',
                                                      'http://anss.org/xmlns/catalog/0.1')
    csstoqml_options['qml_bed_ns'] = safe_pf_get(pf_object, 'BED_NAMESPACE',
                                                 'http://quakeml.org/xmlns/bed/1.2')
    csstoqml_options['qml_bedrt_ns'] = safe_pf_get(pf_object, 'BEDRT_NAMESPACE',
                                                   'http://quakeml.org/xmlns/bed-rt/1.2')
    csstoqml_options['info_description'] = safe_pf_get(pf_object, 'event_info_description', None)
    csstoqml_options['info_comment'] = safe_pf_get(pf_object, 'event_info_comment', None)
    csstoqml_options['preferred_magtypes'] = safe_pf_get(pf_object, 'preferred_magtypes', [])

    for x in ['add_origin', 'add_fplane', 'add_stamag', 'add_mt', 'add_detection', 'extend_anss_catalog',
              'add_arrival', 'add_magnitude']:
        # If user sent preference, use user value over the one defined in pf file
        if override_options and x in override_options:
            logger.debug("User requested to override \"{}\" option".format(x))
            _val = override_options[x]
        else:
            _val = safe_pf_get(pf_object, x, 'true')
        if isinstance(_val, bool):
            csstoqml_options[x] = _val
        else:
            csstoqml_options[x] = stock.yesno(_val)

    return csstoqml_options


def event2qml(pf_file, database, event_id=None,
              includeallorigins=None,
              dbreader_options=None, csstoqml_options=None, log_level="DEBUG",
              order_by=None):
    """Function to return xml data per user request
    :param pf_file: path to parameter file which are the defaults
    :param database: path to database containing all event info
    :param event_id: event ID to grab
    :param dbreader_options: User requested options to override default db reader options
    :param csstoqml_options: User requested options to override default csstoqml options
    :param log_level:
    :return:
    :param pf_file:
    :param database:
    :param int event_id: event ID to return
    :param dict dbreader_options:
    :param dict csstoqml_options:
    :param log_level:
    :return:
    """
    logger.debug("Parameter File: {}".format(pf_file))
    logger.debug('Log level: {}'.format(log_level))
    logger.debug("Database: {}".format(database))
    if event_id:
        logger.info("Requested event_id: {}".format(event_id))
        event_id = int(event_id)

    if not dbreader_options:
        dbreader_options = build_dbreader_options(pf_file)

    logging.debug('Initializing database reader')
    reader = DatabaseReader(database, **dbreader_options)

    csstoqml_options_from_pf_file = build_csstoqml_options(pf_file, override_options=csstoqml_options)

    logging.debug('Initializing CSS3.0 to QuakeML converter')
    converter = Css2Qml(reader, includeallorigins=includeallorigins, **csstoqml_options_from_pf_file)

    if event_id:
        # Get event will check if event exists and raise EventNotFound exception if not found
        reader.get_event(event_id)
        evids = [event_id]
    else:
        evids = reader.get_evids()
        evids = sorted(evids)

    if not evids:
        raise exceptions.NoEventDataFound()

    logging.debug('Dumping to QuakeML and encoding as XML')
    return xmlencode(converter.dump(evids, order_by=order_by))


# pylint: disable=too-many-locals,too-many-statements
def main(argv=()):
    """
    Command-line interface for event2qml.

    Returns
    -------
    int:
        0: successful termination,
        1: processing error,
        2: argument parsing error
    """
    parser = _argparser()

    if len(argv) == 1:
        parser.print_help()
        return 2
    args = parser.parse_args(argv[1:])

    # Pull values from ParameterFile
    pf_file = stock.pffiles(args.pf)[-1]
    logger.debug("Parameter file to use [%s]" % pf_file)
    pf_object = open_verify_pf(pf_file, 1472083200)

    dbreader_options = build_dbreader_options(pf_object)
    csstoqml_options = build_csstoqml_options(pf_object)

    results = event2qml(args.pf, args.database, args.evid, dbreader_options,
                        csstoqml_options, log_level=args.log)

    if args.output_file:

        append_to_output_file = stock.yesno(safe_pf_get(pf_object, 'append_to_output_file', 'false'))
        if append_to_output_file:
            logger.info('Appending to: %s' % args.output_file)
            mode = 'a'
        else:
            logger.info('Writing to: %s' % args.output_file)
            mode = 'w'

        with open(args.output_file, mode) as output_file_object:
            output_file_object.write(results)

    else:
        logger.debug('Print Event in QuakeML format to stdout')
        print(results)  # pylint: disable=superfluous-parens

    if VALIDATION_POSSIBLE:
        logger.info('Attempting validation against schema: ' +
                    os.path.basename(args.schema))

        valid = 'unknown'
        schema_file = args.schema
        logger.debug('Looking for file: %s' % schema_file)

        if not os.path.exists(schema_file):
            package_root = os.path.abspath(os.path.dirname(__file__))
            schema_file = os.path.join(package_root, 'schemas',
                                       _DEFAULT_SCHEMA)

        if os.path.exists(schema_file):

            logger.debug('Validating against: %s' % schema_file)
            xmldoc = etree.fromstring(results.encode())
            valid = etree.RelaxNG(file=schema_file).validate(xmldoc)

            logger.info('%s validation result: %s'
                        % (os.path.basename(args.schema), valid))

        else:
            logger.error('Could not find schema definition: %s' % schema_file)
            return 1

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))
