import pytest
import urllib3
from obspy import UTCDateTime
from obspy.clients.fdsn import Client
from obspy.clients.fdsn.header import FDSNException, FDSNNoDataException

from tests.deployment_validation import util


ip_400_matrix = [{},
                 {'mindepth': 0.1},
                 {'mindepth': 0.1, 'maxdepth': 1000000},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time'},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00'},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00', 'minlatitude': -90},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00', 'minlatitude': -90, 'maxlatitude': 90},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00', 'minlatitude': -90, 'maxlatitude': 90,
                  'minlongitude': -180},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00', 'minlatitude': -90, 'maxlatitude': 90,
                  'minlongitude': -180},
                 {'mindepth': 0.1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
                  'starttime': '2000-01-01 00:00:00', 'minlatitude': -90, 'maxlatitude': 90,
                  'maxlongitude': 180}]


@pytest.mark.parametrize("query_params", ip_400_matrix)
def test_event_query_response_codes(server, query_params):
    api = util.load_server(server, None)
    url = urllib3.util.parse_url(api)
    host = f"{url.scheme}://{url.host}"
    if url.port:
        host = f"{host}:{url.port}"
    client = Client(host)
    with pytest.raises(FDSNException):
        client.get_events(**query_params)


all_params = [{'mindepth': 1, 'maxdepth': 1000000, 'minmagnitude': 1, 'maxmagnitude': 10, 'orderby': 'time',
               'starttime': '2000-01-01 00:00:00', 'minlatitude': -90, 'maxlatitude': 90,
               'maxlongitude': 180, 'minlongitude': -180}]



def _get_url(server):
    api = util.load_server(server, None)
    url = urllib3.util.parse_url(api)
    host = f"{url.scheme}://{url.host}"
    if url.port:
        host = f"{host}:{url.port}"
    return host


@pytest.mark.parametrize("query_params", all_params)
def test_depth_query(server, query_params):
    test_mindepth = 11  # This is in km
    query_params['mindepth'] = test_mindepth
    client = Client(_get_url(server))

    event_catalog = client.get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.depth >= test_mindepth * 1000

    test_maxdepth = 12
    query_params['maxdepth'] = test_maxdepth
    query_params['mindepth'] = 1

    event_catalog = client.get_events(**query_params)
    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.depth <= test_maxdepth * 1000


@pytest.mark.parametrize("query_params", all_params)
def test_latitude_query(server, query_params):
    query_params['minlatitude'] = 34
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.latitude >= 34

    query_params['minlatitude'] = -90
    query_params['maxlatitude'] = 34
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.latitude <= 34


@pytest.mark.parametrize("query_params", all_params)
def test_longitude_query(server, query_params):
    query_params['minlongitude'] = -116.3
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.longitude >= -116.3

    query_params['minlongitude'] = -180
    query_params['maxlongitude'] = -116.4
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for origin in event.origins:
            # depth value on returned origins is in meters
            assert origin.longitude <= -116.4


@pytest.mark.parametrize("query_params", all_params)
def test_magnitude_query(server, query_params):
    query_params['minmagnitude'] = 2.5
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    count_2_5 = len(event_catalog.events)
    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for mag in event.magnitudes:
            assert mag.mag >= 2.5

    #query_params['minmagnitude'] = 2.6
    #event_catalog = Client(_get_url(server)).get_events(**query_params)
    #count_2_6 = len(event_catalog.events)

    #assert len(event_catalog.events) > 0
    #for event in event_catalog.events:
    #    for mag in event.magnitudes:
    #        assert mag.mag >= 2.6

    # count_2_6 should be lower than count_2_5
    #assert count_2_6 < count_2_5

    query_params['minmagnitude'] = 1
    query_params['maxmagnitude'] = 2.1
    event_catalog = Client(_get_url(server)).get_events(**query_params)

    assert len(event_catalog.events) > 0
    for event in event_catalog.events:
        for mag in event.magnitudes:
            assert mag.mag <= 2.1
