"""
Run these tests once a patch to deployed to an Antelope server to validate deployment
"""
import requests
from urllib.parse import urlencode
from http import HTTPStatus
from urllib.parse import urljoin, urlencode
import tempfile
import subprocess
from xml.etree import ElementTree as ET
import pytest
from .util import load_server


WADL_ENDPOINT = 'application.wadl'
VERSION_ENDPOINT = 'version'


dataselect_param_matrix = [({'starttime': '01-12-2016 03:28:00', 'endtime': '01-12-2016 03:31:00',
                             'station': 'CSL.*', 'channel': '.*', 'location': '.*',
                             'nodata': '404'}, HTTPStatus.OK),
                           ({'starttime': '01-12-2016 03:28:00', 'endtime': '01-12-2016 03:31:00',
                             'station': 'RRSP', 'channel': 'H.*', 'location': '.*',
                             'nodata': '404'}, HTTPStatus.OK),
                           ({'starttime': '1/13/16 06:02:07.76454', 'endtime': ' 1/13/16 06:17:09.26954',
                             'station': 'RRSP', 'channel': 'HHE', 'location': '.*',
                             'nodata': '404'}, HTTPStatus.OK),
                           ({'starttime': '01-12-2016 03:28:00', 'endtime': '01-12-2016 03:31:00',
                             'station': 'RRSP', 'channel': '.*', 'location': '.*',
                             'nodata': '204'}, HTTPStatus.OK),
                           ({'mindepth': 1}, HTTPStatus.BAD_REQUEST),
                           ({'starttime': '2020-01-01',
                             'station': 'BALD', 'channel': '.*', 'location': '.*',
                             'nodata': '204'}, HTTPStatus.NO_CONTENT),
                           ({'starttime': '2020-01-01',
                             'station': 'FOST', 'channel': '.*', 'location': '.*',
                             'nodata': '404'}, HTTPStatus.NOT_FOUND),
                           ({'starttime': '01-13-16 16:50:02.36839',
                             'station': 'KNW', 'channel': 'HHZ', 'location': '.*',
                             'nodata': '204'}, HTTPStatus.FORBIDDEN)]


def test_version(server):
    api = load_server(server, 'dataselect/')
    _url = urljoin(api, VERSION_ENDPOINT)
    resp = requests.get(_url)
    assert resp.status_code == HTTPStatus.OK

def test_wadl(server):
    api = load_server(server, 'dataselect/')
    _url = urljoin(api, WADL_ENDPOINT)
    resp = requests.get(_url)
    assert resp.status_code == HTTPStatus.OK


@pytest.mark.parametrize("query_data", dataselect_param_matrix)
def test_validate_sunnyday_data(server, query_data):
    api = load_server(server, 'dataselect/')
    _url = api + 'query?' + urlencode(query_data[0])
    resp = requests.get(_url)
    assert resp.status_code == query_data[1]

    if resp.status_code == HTTPStatus.OK:
        with tempfile.NamedTemporaryFile(mode='wb+') as f:
            f.write(resp.content)
            f.file.flush()
            assert subprocess.check_output('seeddump {}'.format(f.name), shell=True)


@pytest.mark.parametrize("query_data", dataselect_param_matrix)
def test_validate_sunnyday_data_post(server, query_data):
    api = load_server(server, 'dataselect/')
    _post_url = api + 'query'
    resp = requests.post(_post_url, json=query_data[0])
    assert resp.status_code == query_data[1]

    if resp.status_code == HTTPStatus.OK:
        with tempfile.NamedTemporaryFile(mode='wb+') as f:
            f.write(resp.content)
            f.file.flush()
            assert subprocess.check_output('seeddump {}'.format(f.name), shell=True)
